"use strict";
const path = require("path");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const commands_1 = require("../commands");
const globalPlugin = require("../index");
exports.defaultPlugin = 'core';
exports.knownPlugins = [exports.defaultPlugin, 'cloud', 'cordova'];
exports.PREFIX = 'cli-plugin-';
exports.ERROR_PLUGIN_NOT_FOUND = 'PLUGIN_NOT_FOUND';
exports.ERROR_PLUGIN_INVALID = 'PLUGIN_INVALID';
/**
 * Synchronously load a plugin
 */
function loadPlugin(projectDir, name) {
    let m;
    try {
        var mPath = require.resolve(path.join(projectDir, 'node_modules', '@ionic', `${exports.PREFIX}${name}`));
        m = require(mPath);
    }
    catch (e) {
        if (e.code === 'MODULE_NOT_FOUND') {
            throw exports.ERROR_PLUGIN_NOT_FOUND;
        }
        throw e;
    }
    return m;
}
exports.loadPlugin = loadPlugin;
/**
 * Get inputs and command class based on arguments supplied
 */
function resolvePlugin(projectDir, argv) {
    let pluginName;
    let inputs = [];
    /**
     * If this module's primary namespace has the command then use it.
     */
    const ionicNamespace = new commands_1.IonicNamespace();
    const isGlobalCmd = ionicNamespace.getCommands().has(argv[0]);
    if (isGlobalCmd || argv.length === 0) {
        return [
            globalPlugin,
            argv
        ];
    }
    /**
     * If the first arguement supplied contains a ':' then
     * it is assumed that this is calling a command in another
     * namespace. <namespaceName>:<commandName>
     *
     * Else it is likely a core command that should be loaded
     * from the 'core' plugin.
     */
    if (argv.length > 0 && argv[0].includes(':')) {
        let [firstArg, ...restOfArguments] = argv;
        [pluginName, firstArg] = firstArg.split(':');
        inputs = [firstArg, ...restOfArguments];
    }
    else if (exports.knownPlugins.includes(argv[0])) {
        [pluginName, ...inputs] = argv;
    }
    else {
        pluginName = exports.defaultPlugin;
        inputs = argv;
    }
    /**
     * Load the plugin using the pluginName provided
     */
    try {
        return [
            loadPlugin(projectDir, pluginName),
            inputs
        ];
    }
    catch (e) {
        /**
         * If plugin is not found then lets make a recommendation
         * based on whether we know the plugin exists.
         */
        if (e === exports.ERROR_PLUGIN_NOT_FOUND) {
            if (exports.knownPlugins.includes(pluginName)) {
                throw new cli_utils_1.FatalException('This plugin is not currently installed. Please execute the following to install it.\n'
                    + `    ${chalk.bold(`npm install ${exports.PREFIX}${pluginName}`)}`);
            }
            else {
                throw new cli_utils_1.FatalException(`Unknown plugin: ${chalk.bold(exports.PREFIX + pluginName)}.`);
            }
        }
        throw e;
    }
}
exports.resolvePlugin = resolvePlugin;
